/**
 ****************************************************************************************
 *
 * @file app_custom.h
 *
 * @brief Device Information Application Module Entry point
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */

#ifndef APP_CUSTOM_H_
#define APP_CUSTOM_H_

/**
 ****************************************************************************************
 * @addtogroup APP
 *
 * @brief Device Information Application Module Entry point.
 *
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"     // SW Configuration

#include <stdint.h>
#include <string.h>

/*
 * DEFINES
 ****************************************************************************************
 */

/// Manufacturer Name Value
#define APP_CUSTOM_REMOTE_IO_LEN        (1)

#if (BLE_APP_HID)
#define APP_CUSTOM_FEATURES             (CUSTOM_MANUFACTURER_NAME_CHAR_SUP |\
                                      CUSTOM_MODEL_NB_STR_CHAR_SUP      |\
                                      CUSTOM_FIRM_REV_STR_CHAR_SUP      |\
                                      CUSTOM_SW_REV_STR_CHAR_SUP)
#else
#define APP_CUSTOM_FEATURES             (CUSTOM_ALL_FEAT_SUP)
#endif //(BLE_APP_HID)


/*
 * GLOBAL VARIABLES DECLARATION
 ****************************************************************************************
 */

/// Table of message handlers
extern const struct ke_state_handler app_custom_table_handler;

/*
 * GLOBAL FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize Device Information Service Application
 ****************************************************************************************
 */
void app_custom_init(void);

/**
 ****************************************************************************************
 * @brief Add a Device Information Service instance in the DB
 ****************************************************************************************
 */
void app_custom_add_customs(void);

/**
 ****************************************************************************************
 * @brief Enable the Device Information Service
 ****************************************************************************************
 */
void app_custom_enable_prf(uint16_t conhdl);


/// @} APP

#endif //APP_CUSTOM_H_
