/**
 ****************************************************************************************
 *
 * @file app_f000.c
 *
 * @brief findt Application Module entry point
 *
 * @auth  gang.cheng
 *
 * @date  2016.05.31
 *
 * Copyright (C) Beken 2009-2016
 *
 *
 ****************************************************************************************
 */
#ifndef APP_MIDI_H_
#define APP_MIDI_H_
/**
 ****************************************************************************************
 * @addtogroup APP
 * @ingroup RICOW
 *
 * @brief f000 Application Module entry point
 *
 * @{
 ****************************************************************************************
 */
/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h" // SW configuration

#include <stdint.h>  // Standard Integer Definition
#include "ke_task.h" // Kernel Task Definition

/*
 * STRUCTURES DEFINITION
 ****************************************************************************************
 */

/// MIDI Application Module Environment Structure
struct app_midi_env_tag
{
    /// Connection handle
    uint8_t conidx;
    /// Current MIDI data
    uint8_t midi_payload;
};
/*
 * GLOBAL VARIABLES DECLARATIONS
 ****************************************************************************************
 */

/// MIDI Application environment
extern struct app_midi_env_tag app_midi_env;

/// Table of message handlers
extern const struct ke_state_handler app_midi_table_handler;
/*
 * FUNCTIONS DECLARATION
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 *
 * MIDI Application Functions
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Initialize MIDI Application Module
 ****************************************************************************************
 */
void app_midi_init(void);
/**
 ****************************************************************************************
 * @brief Add a MIDI Service instance in the DB
 ****************************************************************************************
 */
void app_midi_add_midis(void);
/**
 ****************************************************************************************
 * @brief Enable the MIDI Service
 ****************************************************************************************
 */
void app_midi_enable_prf(uint8_t conidx);
/**
 ****************************************************************************************
 * @brief Send a MIDI data
 ****************************************************************************************
 */
void app_midi_send_payload(uint8_t *buf, uint8_t len);

#endif // APP_MIDI_H_
