/**
 ****************************************************************************************
 *
 * @file custom.h
 *
 * @brief Header file - Device Information Service Server.
 *
 * Copyright (C) RivieraWaves 2009-2016
 *
 *
 ****************************************************************************************
 */

#ifndef CUSTOM_H_
#define CUSTOM_H_

/**
 ****************************************************************************************
 * @addtogroup CUSTOM Device Information Service Server
 * @ingroup CUSTOM
 * @brief Device Information Service Server
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include "rwip_config.h"
#if (BLE_CUSTOM_SERVER)
#include "prf_types.h"
#include "prf.h"

/*
 * DEFINES
 ****************************************************************************************
 */


#define CUSTOM_IDX_MAX        (1)

/*
 * ENUMERATIONS
 ****************************************************************************************
 */

enum
{		
  
	ATT_USER_SERVER_CUSTOM               = ATT_UUID_16(0xc0de),	  
	ATT_USER_SERVER_CHAR_TEST1           = ATT_UUID_16(0xc101),		
};

/// Possible states of the CUSTOM task
enum
{
    /// Idle state
    CUSTOM_IDLE,
    /// Busy state
    CUSTOM_BUSY,
    /// Number of defined states.
    CUSTOM_STATE_MAX
};

/// CUSTOM Attributes database handle list
enum custom_att_db_handles
{
    CUSTOM_IDX_SVC,

    CUSTOM_IDX_REMOTE_IO_CHAR,
    CUSTOM_IDX_REMOTE_IO_VAL,
  
    CUSTOM_IDX_NB,
};

///Attribute Table Indexes
enum custom_info
{
    /// Manufacturer Name
    CUSTOM_REMOTE_IO_STATUS,

    CUSTOM_CHAR_MAX,
};

///Database Configuration Flags
enum custom_features
{
    ///Indicate if Manufacturer Name String Char. is supported
    CUSTOM_MANUFACTURER_NAME_CHAR_SUP       = 0x0001,

    ///All features are supported
    CUSTOM_ALL_FEAT_SUP                     = 0x01FF,
};


/// Value element
struct custom_val_elmt
{
    /// list element header
    struct co_list_hdr hdr;
    /// value identifier
    uint8_t value;
    /// value length
    uint8_t length;
    /// value data
    uint8_t data[__ARRAY_EMPTY];
};

///Device Information Service Server Environment Variable
struct custom_env_tag
{
    /// profile environment
    prf_env_t prf_env;
    /// List of values set by application
    struct co_list values;
    /// Service Attribute Start Handle
    uint16_t start_hdl;
    /// Services features
    uint16_t features;
    /// Last requested value
    uint8_t  req_val;
    /// Last connection index which request value
    uint8_t  req_conidx;

    /// CUSTOM task state
    ke_state_t state[CUSTOM_IDX_MAX];
};

/*
 * GLOBAL VARIABLE DECLARATIONS
 ****************************************************************************************
 */

/*
 * FUNCTION DECLARATIONS
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @brief Retrieve CUSTOM service profile interface
 *
 * @return CUSTOM service profile interface
 ****************************************************************************************
 */
const struct prf_task_cbs* customs_prf_itf_get(void);


/**
 ****************************************************************************************
 * @brief Check if an attribute shall be added or not in the database
 *
 * @param features CUSTOM features
 *
 * @return Feature config flag
 ****************************************************************************************
 */
uint32_t custom_compute_cfg_flag(uint16_t features);

/**
 ****************************************************************************************
 * @brief Check if the provided value length matches characteristic requirements
 * @param char_code Characteristic Code
 * @param val_len   Length of the Characteristic value
 *
 * @return status if value length is ok or not
 ****************************************************************************************
 */
uint8_t custom_check_val_len(uint8_t char_code, uint8_t val_len);

/**
 ****************************************************************************************
 * @brief Retrieve handle attribute from value
 *
 * @param[in] env   Service environment variable
 * @param[in] value Value to search
 *
 * @return Handle attribute from value
 ****************************************************************************************
 */
uint16_t custom_value_to_handle(struct custom_env_tag* env, uint8_t value);

/**
 ****************************************************************************************
 * @brief Retrieve value from attribute handle
 *
 * @param[in] env    Service environment variable
 * @param[in] handle Attribute handle to search
 *
 * @return  Value from attribute handle
 ****************************************************************************************
 */
uint8_t custom_handle_to_value(struct custom_env_tag* env, uint16_t handle);

/*
 * TASK DESCRIPTOR DECLARATIONS
 ****************************************************************************************
 */
extern const struct ke_state_handler custom_default_handler;

#endif //BLE_CUSTOM_SERVER

/// @} CUSTOM

#endif // CUSTOM_H_
