/**
 ****************************************************************************************
 *
 * @file uart_tp.h
 *
 * @brief Header file - UART transparent Service Server Role
 *
 * Copyright (C) beken 2009-2015
 *
 *
 ****************************************************************************************
 */
#ifndef _MIDI_H_
#define _MIDI_H_

/**
 ****************************************************************************************
 * @addtogroup  UART transparent 'Profile' Server
 * @ingroup UART transparent
 * @brief UART transparent 'Profile' Server
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"
#include "rwprf_config.h"

#if (BLE_MIDI_SERVER)

#include "midi_task.h"
#include "atts.h"
#include "prf_types.h"
#include "prf.h"


/*
 * DEFINES
 ****************************************************************************************
 */

#define MIDI_CFG_FLAG_MANDATORY_MASK     (0x3F)
#define MIDI_CFG_FLAG_NTF_SUP_MASK       (0x08)

#define MIDI_SERVICE_UUID                (0x03B8)
#define MIDI_CHAR_UUID                   (0x7772)

enum
{		
		ATT_MIDI_SERVER_SVC 						     = ATT_UUID_16(MIDI_SERVICE_UUID),
	  
		ATT_MIDI_SERVER_CHAR                 = ATT_UUID_16(MIDI_CHAR_UUID),
};

/// MIDI Service Attributes Indexes
enum
{
	MIDI_IDX_SVC,

	MIDI_DATA_IO_CHAR,
	MIDI_DATA_IO_PAYLOAD,
	MIDI_DATA_IO_NTF_CFG,

	MIDI_DATA_IO_IDX_NB,
};

/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */


/// MIDI 'Profile' Server environment variable
struct midi_env_tag
{
    /// profile environment
    prf_env_t prf_env;
   
    /// On-going operation
    struct ke_msg * operation;
    /// UART transparent Services Start Handle
    uint16_t start_hdl;
    /// Level of the FFF1
    uint8_t uart_tx_payload[UART_TX_PAYLOAD_LEN];
	
    uint8_t rx_payload[UART_RX_PAYLOAD_LEN];
    /// BASS task state
    ke_state_t state[MIDI_IDX_MAX];
    /// Notification configuration of peer devices.
    uint8_t ntf_cfg[BLE_CONNECTION_MAX];
    /// Database features
    uint8_t features;

};



/**
 ****************************************************************************************
 * @brief Retrieve fff0 service profile interface
 *
 * @return fff0 service profile interface
 ****************************************************************************************
 */
const struct prf_task_cbs* midi_prf_itf_get(void);

uint16_t midi_get_att_handle(uint8_t att_idx);

uint8_t  midi_get_att_idx(uint16_t handle, uint8_t *att_idx);

void midi_notify_payload(struct midi_env_tag* f000_env, struct midi_payload_upd_req const *param);

#endif /* #if (BLE_MIDI_SERVER) */



#endif /*  _MIDI_H_ */



