/**
 ****************************************************************************************
 *
 * @file midi_task.h
 *
 * @brief Header file - MIDI Service Server Role Task.
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */


#ifndef _MIDI_TASK_H_
#define _MIDI_TASK_H_


#include "rwprf_config.h"
#if (BLE_MIDI_SERVER)
//#if 1
#include <stdint.h>
#include "rwip_task.h" // Task definitions
/*
 * DEFINES
 ****************************************************************************************
 */

///Maximum number of MIDI Server task instances
#define MIDI_IDX_MAX         0x01
///Maximal number of UART transparent that can be added in the DB

#define UART_TX_PAYLOAD_LEN  128
#define UART_RX_PAYLOAD_LEN  128
/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */

/// Possible states of the MIDI task
enum midi_state
{
    /// Idle state
    MIDI_IDLE,
    /// busy state
    MIDI_BUSY,
    /// Number of defined states.
    MIDI_STATE_MAX
};

/// Messages for MIDI Server
enum midi_msg_id
{
    /// Start the MIDI Server - at connection used to restore bond data
    MIDI_CREATE_DB_REQ   = TASK_FIRST_MSG(TASK_ID_MIDI),
	
    /// MIDI Update Request
    MIDI_PAYLOAD_UPD_REQ,
    /// Inform APP if MIDI value has been notified or not
    MIDI_PAYLOAD_UPD_RSP,
    /// Inform APP that MIDI Notification Configuration has been changed - use to update bond data
    MIDI_PAYLOAD_NTF_CFG_IND,
	
    MIDI_WRITER_REQ_IND,

    MIDI_PAYLOAD_PERIOD_NTF
	
		
};

/// Features Flag Masks
enum midi_features
{
    /// MIDI Characteristic doesn't support notifications
    MIDI_NTF_NOT_SUP,
    /// MIDI Characteristic support notifications
    MIDI_NTF_SUP,
};
/*
 * APIs Structures
 ****************************************************************************************
 */

/// Parameters for the database creation
struct midi_db_cfg
{
    /// Number of MIDI to add
    uint8_t midi_nb;
    /// Features of each MIDI instance
    uint8_t features;
};

///Parameters of the @ref MIDI_PAYLOAD_UPD_REQ message
struct midi_payload_upd_req
{
    /// MIDI instance
    uint8_t conidx;
	
    uint8_t length;
    /// MIDI data
    uint8_t tx_payload[UART_TX_PAYLOAD_LEN];
};

///Parameters of the @ref MIDI_PAYLOAD_UPD_RSP message
struct midi_payload_upd_rsp
{
    /// status
    uint8_t status;
};

///Parameters of the @ref MIDI_PAYLOAD_NTF_CFG_IND message
struct midi_payload_ntf_cfg_ind
{
    /// connection index
    uint8_t  conidx;
    /// Notification Configuration
    uint8_t  ntf_cfg;
};


/// Parameters of the @ref MIDI_WRITER_REQ_IND message
struct midi_writer_ind
{
    /// Alert level
    uint8_t rx_payload[UART_RX_PAYLOAD_LEN];
	
    uint8_t length;
    /// Connection index
    uint8_t conidx;
};


/*
 * TASK DESCRIPTOR DECLARATIONS
 ****************************************************************************************
 */

extern const struct ke_state_handler midi_default_handler;
#endif // BLE_MIDI_SERVER


#endif /* _MIDI_TASK_H_ */

